@@ -1,103 +1,119 @@
 # The table below is taken from
 # http://www.adobe.com/devnet/opentype/archives/aglfn.txt
 
+from __future__ import print_function, division, absolute_import
+from fontTools.misc.py23 import *
+
 _aglText = """\
-# ###################################################################################
-# Copyright (c) 2003,2005,2006,2007 Adobe Systems Incorporated
-# 
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this documentation file to use, copy, publish, distribute,
-# sublicense, and/or sell copies of the documentation, and to permit
-# others to do the same, provided that:
-# - No modification, editing or other alteration of this document is
-# allowed; and
-# - The above copyright notice and this permission notice shall be
-# included in all copies of the documentation.
-# 
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this documentation file, to create their own derivative works
-# from the content of this document to use, copy, publish, distribute,
-# sublicense, and/or sell the derivative works, and to permit others to do
-# the same, provided that the derived work is not represented as being a
-# copy or version of this document.
-# 
-# Adobe shall not be liable to any party for any loss of revenue or profit
-# or for indirect, incidental, special, consequential, or other similar
-# damages, whether based on tort (including without limitation negligence
-# or strict liability), contract or other legal or equitable grounds even
-# if Adobe has been advised or had reason to know of the possibility of
-# such damages. The Adobe materials are provided on an "AS IS" basis.
-# Adobe specifically disclaims all express, statutory, or implied
-# warranties relating to the Adobe materials, including but not limited to
-# those concerning merchantability or fitness for a particular purpose or
-# non-infringement of any third party rights regarding the Adobe
-# materials.
-# ###################################################################################
+# -----------------------------------------------------------
+# Copyright 2003, 2005-2008, 2010 Adobe Systems Incorporated.
+# All rights reserved.
+#
+# Redistribution and use in source and binary forms, with or
+# without modification, are permitted provided that the
+# following conditions are met:
+#
+# Redistributions of source code must retain the above
+# copyright notice, this list of conditions and the following
+# disclaimer.
+#
+# Redistributions in binary form must reproduce the above
+# copyright notice, this list of conditions and the following
+# disclaimer in the documentation and/or other materials
+# provided with the distribution.
+#
+# Neither the name of Adobe Systems Incorporated nor the names
+# of its contributors may be used to endorse or promote
+# products derived from this software without specific prior
+# written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
+# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
+# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
+# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+# -----------------------------------------------------------
 # Name:          Adobe Glyph List For New Fonts
-# Table version: 1.6
-# Date:          30 Januaury 2006
+# Table version: 1.7
+# Date:          November 6, 2008
+# URL:           http://sourceforge.net/adobe/aglfn/
 #
 # Description:
 #
-#   The Adobe Glyph List For New Fonts (AGLFN) is meant to provide a list of 
-#   base glyph names which are compatible with the AGL specification at
-#   http://partners.adobe.com/asn/developer/type/unicodegn.html.
-#   and which can be used as described in section 6 of that document.
-#
-#   This list comprises the set of glyph names from the AGLv2,0 which map
-#   to via the AGL rules to the semanticly correct Unicode value. For
-#   example, Asmall is omitted as the AGL maps this to the Unicode
-#   Private Use Area value F761, rather than to the Unicode value for the
-#   character "A". "ffi" is also omitted, as the AGL maps this to the
-#   Alphabetic Presentation Forms Area value FB03, rather than
-#   decomposing it to the three-value Unicode sequence 0066,0066,0069.
-#    See section 7.1 of the Unicode Standard 4.0 on this issue.
-#   "arrowvertex" is omitted becuase this now has a real Unicode
-#   character value, and the AGL is now incorrect in mapping this to the 
-#   Private Use Area value  F8E6.
-#
-#  If you do not find an appropriate name for your glyph in this list,
-#  then please refer to section 6 of the document:
-#   http://partners.adobe.com/asn/developer/typeforum/unicodegn.html.
-#
-#	The Unicode values and names are given for convenience.
-#
-# Format: Semicolon-delimited fields:
-#
-#   (1) Standard UV or CUS UV. (4 uppercase hexadecimal digits)
-#
-#   (2) Glyph name. (upper- and lowercase letters, digits)
+# AGLFN (Adobe Glyph List For New Fonts) provides a list of base glyph
+# names that are recommended for new fonts, which are compatible with
+# the AGL (Adobe Glyph List) Specification, and which should be used
+# as described in Section 6 of that document. AGLFN comprises the set
+# of glyph names from AGL that map via the AGL Specification rules to
+# the semantically correct UV (Unicode Value). For example, "Asmall"
+# is omitted because AGL maps this glyph name to the PUA (Private Use
+# Area) value U+F761, rather than to the UV that maps from the glyph
+# name "A." Also omitted is "ffi," because AGL maps this to the
+# Alphabetic Presentation Forms value U+FB03, rather than decomposing
+# it into the following sequence of three UVs: U+0066, U+0066, and
+# U+0069. The name "arrowvertex" has been omitted because this glyph
+# now has a real UV, and AGL is now incorrect in mapping it to the PUA
+# value U+F8E6. If you do not find an appropriate name for your glyph
+# in this list, then please refer to Section 6 of the AGL
+# Specification.
 #
+# Format: three semicolon-delimited fields:
+#   (1) Standard UV or CUS UV--four uppercase hexadecimal digits
+#   (2) Glyph name--upper/lowercase letters and digits
 #   (3) Character names: Unicode character names for standard UVs, and
-#       descriptive names for CUS UVs. (uppercase letters, hyphen, space)
+#       descriptive names for CUS UVs--uppercase letters, hyphen, and
+#       space
 #
-#   The entries are sorted by glyph name in increasing ASCII order; entries
-#   with the same glyph name are sorted in decreasing priority order.
+# The records are sorted by glyph name in increasing ASCII order,
+# entries with the same glyph name are sorted in decreasing priority
+# order, the UVs and Unicode character names are provided for
+# convenience, lines starting with "#" are comments, and blank lines
+# should be ignored.
 #
-#   Lines starting with "#" are comments; blank lines should be ignored.
+# Revision History:
 #
-#   1.6  [30 January 2006]
-#	- Completed work intended in 1.5
+# 1.7 [6 November 2008]
+# - Reverted to the original 1.4 and earlier mappings for Delta,
+#   Omega, and mu.
+# - Removed mappings for "afii" names. These should now be assigned
+#   "uni" names.
+# - Removed mappings for "commaaccent" names. These should now be
+#   assigned "uni" names.
 #
-#   1.5  [23 November 2005]
-#      - removed duplicated block at end of file
-#      - changed mappings:
-#            2206;Delta;INCREMENT changed to 0394;Delta;GREEK CAPITAL LETTER DELTA
-#            2126;Omega;OHM SIGN changed to 03A9;Omega;GREEK CAPITAL LETTER OMEGA
-#            03BC;mu;MICRO SIGN changed to 03BC;mu;GREEK SMALL LETTER MU
-#      - corrected statement above about why ffi is omitted.
-
-#   1.4  [24 September 2003]  Changed version to 1.4, to avoid confusion 
-#		with the AGL 1.3
-#			fixed spelling errors in the header
-#			fully removed arrowvertex, as it is mapped only to a PUA Unicode value in some fonts.
+# 1.6 [30 January 2006]
+# - Completed work intended in 1.5.
 #
-#   1.1  [17 April 2003]  Renamed [Tt]cedilla back to [Tt]commaaccent:
+# 1.5 [23 November 2005]
+# - Removed duplicated block at end of file.
+# - Changed mappings:
+#   2206;Delta;INCREMENT changed to 0394;Delta;GREEK CAPITAL LETTER DELTA
+#   2126;Omega;OHM SIGN changed to 03A9;Omega;GREEK CAPITAL LETTER OMEGA
+#   03BC;mu;MICRO SIGN changed to 03BC;mu;GREEK SMALL LETTER MU
+# - Corrected statement above about why "ffi" is omitted.
 #
-#   1.0  [31 Jan 2003]  Original version. Derived from the AGLv1.2 by:
-#	-  removing the PUA area codes
-#	- removing duplicate Unicode mappings, and 
-#	- renaming tcommaaccent to tcedilla and Tcommaaccent to Tcedilla 
+# 1.4 [24 September 2003]
+# - Changed version to 1.4, to avoid confusion with the AGL 1.3.
+# - Fixed spelling errors in the header.
+# - Fully removed "arrowvertex," as it is mapped only to a PUA Unicode
+#   value in some fonts.
+#
+# 1.1 [17 April 2003]
+# - Renamed [Tt]cedilla back to [Tt]commaaccent.
+#
+# 1.0 [31 January 2003]
+# - Original version.
+# - Derived from the AGLv1.2 by:
+#   removing the PUA area codes;
+#   removing duplicate Unicode mappings; and
+#   renaming "tcommaaccent" to "tcedilla" and "Tcommaaccent" to "Tcedilla"
 #
 0041;A;LATIN CAPITAL LETTER A
 00C6;AE;LATIN CAPITAL LETTER AE
@@ -126,7 +142,7 @@ _aglText = """\
 0044;D;LATIN CAPITAL LETTER D
 010E;Dcaron;LATIN CAPITAL LETTER D WITH CARON
 0110;Dcroat;LATIN CAPITAL LETTER D WITH STROKE
-0394;Delta;GREEK CAPITAL LETTER DELTA
+2206;Delta;INCREMENT
 0045;E;LATIN CAPITAL LETTER E
 00C9;Eacute;LATIN CAPITAL LETTER E WITH ACUTE
 0114;Ebreve;LATIN CAPITAL LETTER E WITH BREVE
@@ -150,7 +166,6 @@ _aglText = """\
 011E;Gbreve;LATIN CAPITAL LETTER G WITH BREVE
 01E6;Gcaron;LATIN CAPITAL LETTER G WITH CARON
 011C;Gcircumflex;LATIN CAPITAL LETTER G WITH CIRCUMFLEX
-0122;Gcommaaccent;LATIN CAPITAL LETTER G WITH CEDILLA
 0120;Gdotaccent;LATIN CAPITAL LETTER G WITH DOT ABOVE
 0048;H;LATIN CAPITAL LETTER H
 25CF;H18533;BLACK CIRCLE
@@ -178,12 +193,10 @@ _aglText = """\
 0134;Jcircumflex;LATIN CAPITAL LETTER J WITH CIRCUMFLEX
 004B;K;LATIN CAPITAL LETTER K
 039A;Kappa;GREEK CAPITAL LETTER KAPPA
-0136;Kcommaaccent;LATIN CAPITAL LETTER K WITH CEDILLA
 004C;L;LATIN CAPITAL LETTER L
 0139;Lacute;LATIN CAPITAL LETTER L WITH ACUTE
 039B;Lambda;GREEK CAPITAL LETTER LAMDA
 013D;Lcaron;LATIN CAPITAL LETTER L WITH CARON
-013B;Lcommaaccent;LATIN CAPITAL LETTER L WITH CEDILLA
 013F;Ldot;LATIN CAPITAL LETTER L WITH MIDDLE DOT
 0141;Lslash;LATIN CAPITAL LETTER L WITH STROKE
 004D;M;LATIN CAPITAL LETTER M
@@ -191,7 +204,6 @@ _aglText = """\
 004E;N;LATIN CAPITAL LETTER N
 0143;Nacute;LATIN CAPITAL LETTER N WITH ACUTE
 0147;Ncaron;LATIN CAPITAL LETTER N WITH CARON
-0145;Ncommaaccent;LATIN CAPITAL LETTER N WITH CEDILLA
 00D1;Ntilde;LATIN CAPITAL LETTER N WITH TILDE
 039D;Nu;GREEK CAPITAL LETTER NU
 004F;O;LATIN CAPITAL LETTER O
@@ -204,7 +216,7 @@ _aglText = """\
 01A0;Ohorn;LATIN CAPITAL LETTER O WITH HORN
 0150;Ohungarumlaut;LATIN CAPITAL LETTER O WITH DOUBLE ACUTE
 014C;Omacron;LATIN CAPITAL LETTER O WITH MACRON
-03A9;Omega;GREEK CAPITAL LETTER OMEGA
+2126;Omega;OHM SIGN
 038F;Omegatonos;GREEK CAPITAL LETTER OMEGA WITH TONOS
 039F;Omicron;GREEK CAPITAL LETTER OMICRON
 038C;Omicrontonos;GREEK CAPITAL LETTER OMICRON WITH TONOS
@@ -219,7 +231,6 @@ _aglText = """\
 0052;R;LATIN CAPITAL LETTER R
 0154;Racute;LATIN CAPITAL LETTER R WITH ACUTE
 0158;Rcaron;LATIN CAPITAL LETTER R WITH CARON
-0156;Rcommaaccent;LATIN CAPITAL LETTER R WITH CEDILLA
 211C;Rfraktur;BLACK-LETTER CAPITAL R
 03A1;Rho;GREEK CAPITAL LETTER RHO
 0053;S;LATIN CAPITAL LETTER S
@@ -267,13 +278,11 @@ _aglText = """\
 0160;Scaron;LATIN CAPITAL LETTER S WITH CARON
 015E;Scedilla;LATIN CAPITAL LETTER S WITH CEDILLA
 015C;Scircumflex;LATIN CAPITAL LETTER S WITH CIRCUMFLEX
-0218;Scommaaccent;LATIN CAPITAL LETTER S WITH COMMA BELOW
 03A3;Sigma;GREEK CAPITAL LETTER SIGMA
 0054;T;LATIN CAPITAL LETTER T
 03A4;Tau;GREEK CAPITAL LETTER TAU
 0166;Tbar;LATIN CAPITAL LETTER T WITH STROKE
 0164;Tcaron;LATIN CAPITAL LETTER T WITH CARON
-0162;Tcommaaccent;LATIN CAPITAL LETTER T WITH CEDILLA
 0398;Theta;GREEK CAPITAL LETTER THETA
 00DE;Thorn;LATIN CAPITAL LETTER THORN
 0055;U;LATIN CAPITAL LETTER U
@@ -319,241 +328,6 @@ _aglText = """\
 00E4;adieresis;LATIN SMALL LETTER A WITH DIAERESIS
 00E6;ae;LATIN SMALL LETTER AE
 01FD;aeacute;LATIN SMALL LETTER AE WITH ACUTE
-2015;afii00208;HORIZONTAL BAR
-0410;afii10017;CYRILLIC CAPITAL LETTER A
-0411;afii10018;CYRILLIC CAPITAL LETTER BE
-0412;afii10019;CYRILLIC CAPITAL LETTER VE
-0413;afii10020;CYRILLIC CAPITAL LETTER GHE
-0414;afii10021;CYRILLIC CAPITAL LETTER DE
-0415;afii10022;CYRILLIC CAPITAL LETTER IE
-0401;afii10023;CYRILLIC CAPITAL LETTER IO
-0416;afii10024;CYRILLIC CAPITAL LETTER ZHE
-0417;afii10025;CYRILLIC CAPITAL LETTER ZE
-0418;afii10026;CYRILLIC CAPITAL LETTER I
-0419;afii10027;CYRILLIC CAPITAL LETTER SHORT I
-041A;afii10028;CYRILLIC CAPITAL LETTER KA
-041B;afii10029;CYRILLIC CAPITAL LETTER EL
-041C;afii10030;CYRILLIC CAPITAL LETTER EM
-041D;afii10031;CYRILLIC CAPITAL LETTER EN
-041E;afii10032;CYRILLIC CAPITAL LETTER O
-041F;afii10033;CYRILLIC CAPITAL LETTER PE
-0420;afii10034;CYRILLIC CAPITAL LETTER ER
-0421;afii10035;CYRILLIC CAPITAL LETTER ES
-0422;afii10036;CYRILLIC CAPITAL LETTER TE
-0423;afii10037;CYRILLIC CAPITAL LETTER U
-0424;afii10038;CYRILLIC CAPITAL LETTER EF
-0425;afii10039;CYRILLIC CAPITAL LETTER HA
-0426;afii10040;CYRILLIC CAPITAL LETTER TSE
-0427;afii10041;CYRILLIC CAPITAL LETTER CHE
-0428;afii10042;CYRILLIC CAPITAL LETTER SHA
-0429;afii10043;CYRILLIC CAPITAL LETTER SHCHA
-042A;afii10044;CYRILLIC CAPITAL LETTER HARD SIGN
-042B;afii10045;CYRILLIC CAPITAL LETTER YERU
-042C;afii10046;CYRILLIC CAPITAL LETTER SOFT SIGN
-042D;afii10047;CYRILLIC CAPITAL LETTER E
-042E;afii10048;CYRILLIC CAPITAL LETTER YU
-042F;afii10049;CYRILLIC CAPITAL LETTER YA
-0490;afii10050;CYRILLIC CAPITAL LETTER GHE WITH UPTURN
-0402;afii10051;CYRILLIC CAPITAL LETTER DJE
-0403;afii10052;CYRILLIC CAPITAL LETTER GJE
-0404;afii10053;CYRILLIC CAPITAL LETTER UKRAINIAN IE
-0405;afii10054;CYRILLIC CAPITAL LETTER DZE
-0406;afii10055;CYRILLIC CAPITAL LETTER BYELORUSSIAN-UKRAINIAN I
-0407;afii10056;CYRILLIC CAPITAL LETTER YI
-0408;afii10057;CYRILLIC CAPITAL LETTER JE
-0409;afii10058;CYRILLIC CAPITAL LETTER LJE
-040A;afii10059;CYRILLIC CAPITAL LETTER NJE
-040B;afii10060;CYRILLIC CAPITAL LETTER TSHE
-040C;afii10061;CYRILLIC CAPITAL LETTER KJE
-040E;afii10062;CYRILLIC CAPITAL LETTER SHORT U
-0430;afii10065;CYRILLIC SMALL LETTER A
-0431;afii10066;CYRILLIC SMALL LETTER BE
-0432;afii10067;CYRILLIC SMALL LETTER VE
-0433;afii10068;CYRILLIC SMALL LETTER GHE
-0434;afii10069;CYRILLIC SMALL LETTER DE
-0435;afii10070;CYRILLIC SMALL LETTER IE
-0451;afii10071;CYRILLIC SMALL LETTER IO
-0436;afii10072;CYRILLIC SMALL LETTER ZHE
-0437;afii10073;CYRILLIC SMALL LETTER ZE
-0438;afii10074;CYRILLIC SMALL LETTER I
-0439;afii10075;CYRILLIC SMALL LETTER SHORT I
-043A;afii10076;CYRILLIC SMALL LETTER KA
-043B;afii10077;CYRILLIC SMALL LETTER EL
-043C;afii10078;CYRILLIC SMALL LETTER EM
-043D;afii10079;CYRILLIC SMALL LETTER EN
-043E;afii10080;CYRILLIC SMALL LETTER O
-043F;afii10081;CYRILLIC SMALL LETTER PE
-0440;afii10082;CYRILLIC SMALL LETTER ER
-0441;afii10083;CYRILLIC SMALL LETTER ES
-0442;afii10084;CYRILLIC SMALL LETTER TE
-0443;afii10085;CYRILLIC SMALL LETTER U
-0444;afii10086;CYRILLIC SMALL LETTER EF
-0445;afii10087;CYRILLIC SMALL LETTER HA
-0446;afii10088;CYRILLIC SMALL LETTER TSE
-0447;afii10089;CYRILLIC SMALL LETTER CHE
-0448;afii10090;CYRILLIC SMALL LETTER SHA
-0449;afii10091;CYRILLIC SMALL LETTER SHCHA
-044A;afii10092;CYRILLIC SMALL LETTER HARD SIGN
-044B;afii10093;CYRILLIC SMALL LETTER YERU
-044C;afii10094;CYRILLIC SMALL LETTER SOFT SIGN
-044D;afii10095;CYRILLIC SMALL LETTER E
-044E;afii10096;CYRILLIC SMALL LETTER YU
-044F;afii10097;CYRILLIC SMALL LETTER YA
-0491;afii10098;CYRILLIC SMALL LETTER GHE WITH UPTURN
-0452;afii10099;CYRILLIC SMALL LETTER DJE
-0453;afii10100;CYRILLIC SMALL LETTER GJE
-0454;afii10101;CYRILLIC SMALL LETTER UKRAINIAN IE
-0455;afii10102;CYRILLIC SMALL LETTER DZE
-0456;afii10103;CYRILLIC SMALL LETTER BYELORUSSIAN-UKRAINIAN I
-0457;afii10104;CYRILLIC SMALL LETTER YI
-0458;afii10105;CYRILLIC SMALL LETTER JE
-0459;afii10106;CYRILLIC SMALL LETTER LJE
-045A;afii10107;CYRILLIC SMALL LETTER NJE
-045B;afii10108;CYRILLIC SMALL LETTER TSHE
-045C;afii10109;CYRILLIC SMALL LETTER KJE
-045E;afii10110;CYRILLIC SMALL LETTER SHORT U
-040F;afii10145;CYRILLIC CAPITAL LETTER DZHE
-0462;afii10146;CYRILLIC CAPITAL LETTER YAT
-0472;afii10147;CYRILLIC CAPITAL LETTER FITA
-0474;afii10148;CYRILLIC CAPITAL LETTER IZHITSA
-045F;afii10193;CYRILLIC SMALL LETTER DZHE
-0463;afii10194;CYRILLIC SMALL LETTER YAT
-0473;afii10195;CYRILLIC SMALL LETTER FITA
-0475;afii10196;CYRILLIC SMALL LETTER IZHITSA
-04D9;afii10846;CYRILLIC SMALL LETTER SCHWA
-200E;afii299;LEFT-TO-RIGHT MARK
-200F;afii300;RIGHT-TO-LEFT MARK
-200D;afii301;ZERO WIDTH JOINER
-066A;afii57381;ARABIC PERCENT SIGN
-060C;afii57388;ARABIC COMMA
-0660;afii57392;ARABIC-INDIC DIGIT ZERO
-0661;afii57393;ARABIC-INDIC DIGIT ONE
-0662;afii57394;ARABIC-INDIC DIGIT TWO
-0663;afii57395;ARABIC-INDIC DIGIT THREE
-0664;afii57396;ARABIC-INDIC DIGIT FOUR
-0665;afii57397;ARABIC-INDIC DIGIT FIVE
-0666;afii57398;ARABIC-INDIC DIGIT SIX
-0667;afii57399;ARABIC-INDIC DIGIT SEVEN
-0668;afii57400;ARABIC-INDIC DIGIT EIGHT
-0669;afii57401;ARABIC-INDIC DIGIT NINE
-061B;afii57403;ARABIC SEMICOLON
-061F;afii57407;ARABIC QUESTION MARK
-0621;afii57409;ARABIC LETTER HAMZA
-0622;afii57410;ARABIC LETTER ALEF WITH MADDA ABOVE
-0623;afii57411;ARABIC LETTER ALEF WITH HAMZA ABOVE
-0624;afii57412;ARABIC LETTER WAW WITH HAMZA ABOVE
-0625;afii57413;ARABIC LETTER ALEF WITH HAMZA BELOW
-0626;afii57414;ARABIC LETTER YEH WITH HAMZA ABOVE
-0627;afii57415;ARABIC LETTER ALEF
-0628;afii57416;ARABIC LETTER BEH
-0629;afii57417;ARABIC LETTER TEH MARBUTA
-062A;afii57418;ARABIC LETTER TEH
-062B;afii57419;ARABIC LETTER THEH
-062C;afii57420;ARABIC LETTER JEEM
-062D;afii57421;ARABIC LETTER HAH
-062E;afii57422;ARABIC LETTER KHAH
-062F;afii57423;ARABIC LETTER DAL
-0630;afii57424;ARABIC LETTER THAL
-0631;afii57425;ARABIC LETTER REH
-0632;afii57426;ARABIC LETTER ZAIN
-0633;afii57427;ARABIC LETTER SEEN
-0634;afii57428;ARABIC LETTER SHEEN
-0635;afii57429;ARABIC LETTER SAD
-0636;afii57430;ARABIC LETTER DAD
-0637;afii57431;ARABIC LETTER TAH
-0638;afii57432;ARABIC LETTER ZAH
-0639;afii57433;ARABIC LETTER AIN
-063A;afii57434;ARABIC LETTER GHAIN
-0640;afii57440;ARABIC TATWEEL
-0641;afii57441;ARABIC LETTER FEH
-0642;afii57442;ARABIC LETTER QAF
-0643;afii57443;ARABIC LETTER KAF
-0644;afii57444;ARABIC LETTER LAM
-0645;afii57445;ARABIC LETTER MEEM
-0646;afii57446;ARABIC LETTER NOON
-0648;afii57448;ARABIC LETTER WAW
-0649;afii57449;ARABIC LETTER ALEF MAKSURA
-064A;afii57450;ARABIC LETTER YEH
-064B;afii57451;ARABIC FATHATAN
-064C;afii57452;ARABIC DAMMATAN
-064D;afii57453;ARABIC KASRATAN
-064E;afii57454;ARABIC FATHA
-064F;afii57455;ARABIC DAMMA
-0650;afii57456;ARABIC KASRA
-0651;afii57457;ARABIC SHADDA
-0652;afii57458;ARABIC SUKUN
-0647;afii57470;ARABIC LETTER HEH
-06A4;afii57505;ARABIC LETTER VEH
-067E;afii57506;ARABIC LETTER PEH
-0686;afii57507;ARABIC LETTER TCHEH
-0698;afii57508;ARABIC LETTER JEH
-06AF;afii57509;ARABIC LETTER GAF
-0679;afii57511;ARABIC LETTER TTEH
-0688;afii57512;ARABIC LETTER DDAL
-0691;afii57513;ARABIC LETTER RREH
-06BA;afii57514;ARABIC LETTER NOON GHUNNA
-06D2;afii57519;ARABIC LETTER YEH BARREE
-06D5;afii57534;ARABIC LETTER AE
-20AA;afii57636;NEW SHEQEL SIGN
-05BE;afii57645;HEBREW PUNCTUATION MAQAF
-05C3;afii57658;HEBREW PUNCTUATION SOF PASUQ
-05D0;afii57664;HEBREW LETTER ALEF
-05D1;afii57665;HEBREW LETTER BET
-05D2;afii57666;HEBREW LETTER GIMEL
-05D3;afii57667;HEBREW LETTER DALET
-05D4;afii57668;HEBREW LETTER HE
-05D5;afii57669;HEBREW LETTER VAV
-05D6;afii57670;HEBREW LETTER ZAYIN
-05D7;afii57671;HEBREW LETTER HET
-05D8;afii57672;HEBREW LETTER TET
-05D9;afii57673;HEBREW LETTER YOD
-05DA;afii57674;HEBREW LETTER FINAL KAF
-05DB;afii57675;HEBREW LETTER KAF
-05DC;afii57676;HEBREW LETTER LAMED
-05DD;afii57677;HEBREW LETTER FINAL MEM
-05DE;afii57678;HEBREW LETTER MEM
-05DF;afii57679;HEBREW LETTER FINAL NUN
-05E0;afii57680;HEBREW LETTER NUN
-05E1;afii57681;HEBREW LETTER SAMEKH
-05E2;afii57682;HEBREW LETTER AYIN
-05E3;afii57683;HEBREW LETTER FINAL PE
-05E4;afii57684;HEBREW LETTER PE
-05E5;afii57685;HEBREW LETTER FINAL TSADI
-05E6;afii57686;HEBREW LETTER TSADI
-05E7;afii57687;HEBREW LETTER QOF
-05E8;afii57688;HEBREW LETTER RESH
-05E9;afii57689;HEBREW LETTER SHIN
-05EA;afii57690;HEBREW LETTER TAV
-05F0;afii57716;HEBREW LIGATURE YIDDISH DOUBLE VAV
-05F1;afii57717;HEBREW LIGATURE YIDDISH VAV YOD
-05F2;afii57718;HEBREW LIGATURE YIDDISH DOUBLE YOD
-05B4;afii57793;HEBREW POINT HIRIQ
-05B5;afii57794;HEBREW POINT TSERE
-05B6;afii57795;HEBREW POINT SEGOL
-05BB;afii57796;HEBREW POINT QUBUTS
-05B8;afii57797;HEBREW POINT QAMATS
-05B7;afii57798;HEBREW POINT PATAH
-05B0;afii57799;HEBREW POINT SHEVA
-05B2;afii57800;HEBREW POINT HATAF PATAH
-05B1;afii57801;HEBREW POINT HATAF SEGOL
-05B3;afii57802;HEBREW POINT HATAF QAMATS
-05C2;afii57803;HEBREW POINT SIN DOT
-05C1;afii57804;HEBREW POINT SHIN DOT
-05B9;afii57806;HEBREW POINT HOLAM
-05BC;afii57807;HEBREW POINT DAGESH OR MAPIQ
-05BD;afii57839;HEBREW POINT METEG
-05BF;afii57841;HEBREW POINT RAFE
-05C0;afii57842;HEBREW PUNCTUATION PASEQ
-02BC;afii57929;MODIFIER LETTER APOSTROPHE
-2105;afii61248;CARE OF
-2113;afii61289;SCRIPT SMALL L
-2116;afii61352;NUMERO SIGN
-202C;afii61573;POP DIRECTIONAL FORMATTING
-202D;afii61574;LEFT-TO-RIGHT OVERRIDE
-202E;afii61575;RIGHT-TO-LEFT OVERRIDE
-200C;afii61664;ZERO WIDTH NON-JOINER
-066D;afii63167;ARABIC FIVE POINTED STAR
-02BD;afii64937;MODIFIER LETTER REVERSED COMMA
 00E0;agrave;LATIN SMALL LETTER A WITH GRAVE
 2135;aleph;ALEF SYMBOL
 03B1;alpha;GREEK SMALL LETTER ALPHA
@@ -684,7 +458,6 @@ _aglText = """\
 011F;gbreve;LATIN SMALL LETTER G WITH BREVE
 01E7;gcaron;LATIN SMALL LETTER G WITH CARON
 011D;gcircumflex;LATIN SMALL LETTER G WITH CIRCUMFLEX
-0123;gcommaaccent;LATIN SMALL LETTER G WITH CEDILLA
 0121;gdotaccent;LATIN SMALL LETTER G WITH DOT ABOVE
 00DF;germandbls;LATIN SMALL LETTER SHARP S
 2207;gradient;NABLA
@@ -730,13 +503,11 @@ _aglText = """\
 0135;jcircumflex;LATIN SMALL LETTER J WITH CIRCUMFLEX
 006B;k;LATIN SMALL LETTER K
 03BA;kappa;GREEK SMALL LETTER KAPPA
-0137;kcommaaccent;LATIN SMALL LETTER K WITH CEDILLA
 0138;kgreenlandic;LATIN SMALL LETTER KRA
 006C;l;LATIN SMALL LETTER L
 013A;lacute;LATIN SMALL LETTER L WITH ACUTE
 03BB;lambda;GREEK SMALL LETTER LAMDA
 013E;lcaron;LATIN SMALL LETTER L WITH CARON
-013C;lcommaaccent;LATIN SMALL LETTER L WITH CEDILLA
 0140;ldot;LATIN SMALL LETTER L WITH MIDDLE DOT
 003C;less;LESS-THAN SIGN
 2264;lessequal;LESS-THAN OR EQUAL TO
@@ -754,7 +525,7 @@ _aglText = """\
 2642;male;MALE SIGN
 2212;minus;MINUS SIGN
 2032;minute;PRIME
-03BC;mu;GREEK SMALL LETTER MU
+00B5;mu;MICRO SIGN
 00D7;multiply;MULTIPLICATION SIGN
 266A;musicalnote;EIGHTH NOTE
 266B;musicalnotedbl;BEAMED EIGHTH NOTES
@@ -762,7 +533,6 @@ _aglText = """\
 0144;nacute;LATIN SMALL LETTER N WITH ACUTE
 0149;napostrophe;LATIN SMALL LETTER N PRECEDED BY APOSTROPHE
 0148;ncaron;LATIN SMALL LETTER N WITH CARON
-0146;ncommaaccent;LATIN SMALL LETTER N WITH CEDILLA
 0039;nine;DIGIT NINE
 2209;notelement;NOT AN ELEMENT OF
 2260;notequal;NOT EQUAL TO
@@ -837,7 +607,6 @@ _aglText = """\
 0155;racute;LATIN SMALL LETTER R WITH ACUTE
 221A;radical;SQUARE ROOT
 0159;rcaron;LATIN SMALL LETTER R WITH CARON
-0157;rcommaaccent;LATIN SMALL LETTER R WITH CEDILLA
 2286;reflexsubset;SUBSET OF OR EQUAL TO
 2287;reflexsuperset;SUPERSET OF OR EQUAL TO
 00AE;registered;REGISTERED SIGN
@@ -850,7 +619,6 @@ _aglText = """\
 0161;scaron;LATIN SMALL LETTER S WITH CARON
 015F;scedilla;LATIN SMALL LETTER S WITH CEDILLA
 015D;scircumflex;LATIN SMALL LETTER S WITH CIRCUMFLEX
-0219;scommaaccent;LATIN SMALL LETTER S WITH COMMA BELOW
 2033;second;DOUBLE PRIME
 00A7;section;SECTION SIGN
 003B;semicolon;SEMICOLON
@@ -873,7 +641,6 @@ _aglText = """\
 03C4;tau;GREEK SMALL LETTER TAU
 0167;tbar;LATIN SMALL LETTER T WITH STROKE
 0165;tcaron;LATIN SMALL LETTER T WITH CARON
-0163;tcommaaccent;LATIN SMALL LETTER T WITH CEDILLA
 2234;therefore;THEREFORE
 03B8;theta;GREEK SMALL LETTER THETA
 03D1;theta1;GREEK THETA SYMBOL
@@ -934,36 +701,38 @@ _aglText = """\
 017C;zdotaccent;LATIN SMALL LETTER Z WITH DOT ABOVE
 0030;zero;DIGIT ZERO
 03B6;zeta;GREEK SMALL LETTER ZETA
+#END
 """
 
 
-AGLError = "AGLError"
+class AGLError(Exception):
+	pass
 
 AGL2UV = {}
 UV2AGL = {}
 
 def _builddicts():
 	import re
-	
+
 	lines = _aglText.splitlines()
-	
+
 	parseAGL_RE = re.compile("([0-9A-F]{4});([A-Za-z_0-9.]+);.*?$")
-	
+
 	for line in lines:
 		if not line or line[:1] == '#':
 			continue
 		m = parseAGL_RE.match(line)
 		if not m:
-			raise AGLError, "syntax error in glyphlist.txt: %s" % repr(line[:20])
+			raise AGLError("syntax error in glyphlist.txt: %s" % repr(line[:20]))
 		unicode = m.group(1)
 		assert len(unicode) == 4
 		unicode = int(unicode, 16)
 		glyphName = m.group(2)
-		if AGL2UV.has_key(glyphName):
+		if glyphName in AGL2UV:
 			# the above table contains identical duplicates
 			assert AGL2UV[glyphName] == unicode
 		else:
 			AGL2UV[glyphName] = unicode
 		UV2AGL[unicode] = glyphName
-	
+
 _builddicts()
