@@ -10,7 +10,7 @@ that drawings don't need to know the details of how outlines are stored.
 
 The most basic pattern is this:
 
-    outline.draw(pen)  # 'outline' draws itself onto 'pen'
+	outline.draw(pen)  # 'outline' draws itself onto 'pen'
 
 Pens can be used to render outlines to the screen, but also to construct
 new outlines. Eg. an outline object can be both a drawable object (it has a
@@ -36,9 +36,11 @@ Coordinates are usually expressed as (x, y) tuples, but generally any
 sequence of length 2 will do.
 """
 
+from __future__ import print_function, division, absolute_import
+from fontTools.misc.py23 import *
 
-__all__ = ["AbstractPen", "BasePen",
-           "decomposeSuperBezierSegment", "decomposeQuadraticSegment"]
+__all__ =  ["AbstractPen", "NullPen", "BasePen",
+			"decomposeSuperBezierSegment", "decomposeQuadraticSegment"]
 
 
 class AbstractPen(object):
@@ -112,6 +114,33 @@ class AbstractPen(object):
 		raise NotImplementedError
 
 
+class NullPen(object):
+
+	"""A pen that does nothing.
+	"""
+
+	def moveTo(self, pt):
+		pass
+
+	def lineTo(self, pt):
+		pass
+
+	def curveTo(self, *points):
+		pass
+
+	def qCurveTo(self, *points):
+		pass
+
+	def closePath(self):
+		pass
+
+	def endPath(self):
+		pass
+
+	def addComponent(self, glyphName, transformation):
+		pass
+
+
 class BasePen(AbstractPen):
 
 	"""Base class for drawing pens. You must override _moveTo, _lineTo and
@@ -221,7 +250,7 @@ class BasePen(AbstractPen):
 		elif n == 0:
 			self.lineTo(points[0])
 		else:
-			raise AssertionError, "can't get there from here"
+			raise AssertionError("can't get there from here")
 
 	def qCurveTo(self, *points):
 		n = len(points) - 1  # 'n' is the number of control points
@@ -269,9 +298,8 @@ def decomposeSuperBezierSegment(points):
 	for i in range(2, n+1):
 		# calculate points in between control points.
 		nDivisions = min(i, 3, n-i+2)
-		d = float(nDivisions)
 		for j in range(1, nDivisions):
-			factor = j / d
+			factor = j / nDivisions
 			temp1 = points[i-1]
 			temp2 = points[i-2]
 			temp = (temp2[0] + factor * (temp1[0] - temp2[0]),
@@ -279,8 +307,8 @@ def decomposeSuperBezierSegment(points):
 			if pt2 is None:
 				pt2 = temp
 			else:
-				pt3 = (0.5 * (pt2[0] + temp[0]),
-					   0.5 * (pt2[1] + temp[1]))
+				pt3 =  (0.5 * (pt2[0] + temp[0]),
+						0.5 * (pt2[1] + temp[1]))
 				bezierSegments.append((pt1, pt2, pt3))
 				pt1, pt2, pt3 = temp, None, None
 	bezierSegments.append((pt1, points[-2], points[-1]))
@@ -312,14 +340,14 @@ def decomposeQuadraticSegment(points):
 class _TestPen(BasePen):
 	"""Test class that prints PostScript to stdout."""
 	def _moveTo(self, pt):
-		print "%s %s moveto" % (pt[0], pt[1])
+		print("%s %s moveto" % (pt[0], pt[1]))
 	def _lineTo(self, pt):
-		print "%s %s lineto" % (pt[0], pt[1])
+		print("%s %s lineto" % (pt[0], pt[1]))
 	def _curveToOne(self, bcp1, bcp2, pt):
-		print "%s %s %s %s %s %s curveto" % (bcp1[0], bcp1[1],
-				bcp2[0], bcp2[1], pt[0], pt[1])
+		print("%s %s %s %s %s %s curveto" % (bcp1[0], bcp1[1],
+				bcp2[0], bcp2[1], pt[0], pt[1]))
 	def _closePath(self):
-		print "closepath"
+		print("closepath")
 
 
 if __name__ == "__main__":
